Class: Tapsoob::DataStream
- Inherits:
-
Object
- Object
- Tapsoob::DataStream
- Defined in:
- lib/tapsoob/data_stream.rb
Direct Known Subclasses
Constant Summary collapse
- DEFAULT_CHUNKSIZE =
1000
Instance Attribute Summary collapse
-
#db ⇒ Object
readonly
Returns the value of attribute db.
-
#options ⇒ Object
readonly
Returns the value of attribute options.
-
#state ⇒ Object
readonly
Returns the value of attribute state.
Class Method Summary collapse
Instance Method Summary collapse
- #complete? ⇒ Boolean
- #encode_rows(rows) ⇒ Object
- #error ⇒ Object
- #error=(val) ⇒ Object
- #fetch(opts = {}) ⇒ Object
- #fetch_data_from_database(params) {|rows| ... } ⇒ Object
- #fetch_data_to_database(params) ⇒ Object
- #fetch_file(dump_path) ⇒ Object
-
#fetch_rows ⇒ Object
keep a record of the average chunksize within the first few hundred thousand records, after chunksize goes below 100 or maybe if offset is > 1000.
- #import_rows(rows) ⇒ Object
- #increment(row_count) ⇒ Object
-
#initialize(db, state, opts = {}) ⇒ DataStream
constructor
A new instance of DataStream.
- #log ⇒ Object
- #max_chunksize_training ⇒ Object
- #order_by(name = nil) ⇒ Object
- #parse_encoded_data(encoded_data, checksum) ⇒ Object
- #string_columns ⇒ Object
- #table ⇒ Object
- #table_name ⇒ Object
- #table_name_sql ⇒ Object
- #to_hash ⇒ Object
- #to_json ⇒ Object
- #update_chunksize_stats ⇒ Object
- #verify_stream ⇒ Object
Constructor Details
#initialize(db, state, opts = {}) ⇒ DataStream
Returns a new instance of DataStream.
11 12 13 14 15 16 17 18 19 20 21 22 |
# File 'lib/tapsoob/data_stream.rb', line 11 def initialize(db, state, opts = {}) @db = db @state = { :offset => 0, :avg_chunksize => 0, :num_chunksize => 0, :total_chunksize => 0 }.merge(state) @state[:chunksize] ||= DEFAULT_CHUNKSIZE @options = opts @complete = false end |
Instance Attribute Details
#db ⇒ Object (readonly)
Returns the value of attribute db.
9 10 11 |
# File 'lib/tapsoob/data_stream.rb', line 9 def db @db end |
#options ⇒ Object (readonly)
Returns the value of attribute options.
9 10 11 |
# File 'lib/tapsoob/data_stream.rb', line 9 def @options end |
#state ⇒ Object (readonly)
Returns the value of attribute state.
9 10 11 |
# File 'lib/tapsoob/data_stream.rb', line 9 def state @state end |
Class Method Details
.factory(db, state, opts) ⇒ Object
270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 |
# File 'lib/tapsoob/data_stream.rb', line 270 def self.factory(db, state, opts) if defined?(Sequel::MySQL) && Sequel::MySQL.respond_to?(:convert_invalid_date_time=) Sequel::MySQL.convert_invalid_date_time = :nil end if state.has_key?(:klass) return eval(state[:klass]).new(db, state, opts) end if Tapsoob::Utils.single_integer_primary_key(db, state[:table_name].to_sym) DataStreamKeyed.new(db, state, opts) else DataStream.new(db, state, opts) end end |
.parse_json(json) ⇒ Object
187 188 189 190 191 |
# File 'lib/tapsoob/data_stream.rb', line 187 def self.parse_json(json) hash = JSON.parse(json).symbolize_keys hash[:state].symbolize_keys! if hash.has_key?(:state) hash end |
Instance Method Details
#complete? ⇒ Boolean
162 163 164 |
# File 'lib/tapsoob/data_stream.rb', line 162 def complete? state[:offset] >= state[:size] end |
#encode_rows(rows) ⇒ Object
142 143 144 |
# File 'lib/tapsoob/data_stream.rb', line 142 def encode_rows(rows) Tapsoob::Utils.base64encode(Marshal.dump(rows)) end |
#error ⇒ Object
33 34 35 |
# File 'lib/tapsoob/data_stream.rb', line 33 def error state[:error] || false end |
#error=(val) ⇒ Object
29 30 31 |
# File 'lib/tapsoob/data_stream.rb', line 29 def error=(val) state[:error] = val end |
#fetch(opts = {}) ⇒ Object
146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 |
# File 'lib/tapsoob/data_stream.rb', line 146 def fetch(opts = {}) opts = (opts.empty? ? { :type => "database", :source => db.uri } : opts) log.debug "DataStream#fetch state -> #{state.inspect}" t1 = Time.now rows = (opts[:type] == "file" ? fetch_file(opts[:source]) : fetch_rows) encoded_data = encode_rows(rows) t2 = Time.now elapsed_time = t2 - t1 state[:offset] += (rows == {} ? 0 : rows[:data].size) [encoded_data, (rows == {} ? 0 : rows[:data].size), elapsed_time] end |
#fetch_data_from_database(params) {|rows| ... } ⇒ Object
166 167 168 169 170 171 172 173 174 175 176 |
# File 'lib/tapsoob/data_stream.rb', line 166 def fetch_data_from_database(params) encoded_data = params[:encoded_data] rows = parse_encoded_data(encoded_data, params[:checksum]) # update local state state.merge!(params[:state].merge(:chunksize => state[:chunksize])) yield rows if block_given? (rows == {} ? 0 : rows[:data].size) end |
#fetch_data_to_database(params) ⇒ Object
178 179 180 181 182 183 184 185 |
# File 'lib/tapsoob/data_stream.rb', line 178 def fetch_data_to_database(params) encoded_data = params[:encoded_data] rows = parse_encoded_data(encoded_data, params[:checksum]) import_rows(rows) (rows == {} ? 0 : rows[:data].size) end |
#fetch_file(dump_path) ⇒ Object
88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 |
# File 'lib/tapsoob/data_stream.rb', line 88 def fetch_file(dump_path) #state[:chunksize] = fetch_chunksize # Read NDJSON format - each line is a separate JSON chunk file_path = File.join(dump_path, "data", "#{table_name}.json") # Parse all chunks and combine them all_data = [] table_name_val = nil header_val = nil types_val = nil File.readlines(file_path).each do |line| chunk = JSON.parse(line.strip) table_name_val ||= chunk["table_name"] header_val ||= chunk["header"] types_val ||= chunk["types"] all_data.concat(chunk["data"]) if chunk["data"] end # Apply skip-duplicates if needed all_data = all_data.uniq if @options[:"skip-duplicates"] state[:size] = all_data.size log.debug "DataStream#fetch_file" rows = { :table_name => table_name_val, :header => header_val, :data => (all_data[state[:offset], state[:chunksize]] || []), :types => types_val } update_chunksize_stats rows end |
#fetch_rows ⇒ Object
keep a record of the average chunksize within the first few hundred thousand records, after chunksize goes below 100 or maybe if offset is > 1000
74 75 76 77 78 79 80 81 82 83 84 85 86 |
# File 'lib/tapsoob/data_stream.rb', line 74 def fetch_rows #state[:chunksize] = fetch_chunksize ds = table.order(*order_by).limit(state[:chunksize], state[:offset]) state[:size] = table.count log.debug "DataStream#fetch_rows SQL -> #{ds.sql}" rows = Tapsoob::Utils.format_data(db, ds.all, :string_columns => string_columns, :schema => db.schema(table_name), :table => table_name ) update_chunksize_stats rows end |
#import_rows(rows) ⇒ Object
207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 |
# File 'lib/tapsoob/data_stream.rb', line 207 def import_rows(rows) columns = rows[:header] data = rows[:data] # Only import existing columns if table.columns.size != columns.size existing_columns = table.columns.map(&:to_s) additional_columns = columns - existing_columns additional_columns_idxs = additional_columns.map { |c| columns.index(c) } additional_columns_idxs.reverse.each do |idx| columns.delete_at(idx) rows[:types].delete_at(idx) end data.each_index { |didx| additional_columns_idxs.reverse.each { |idx| data[didx].delete_at(idx) } } end # Decode blobs if rows.has_key?(:types) && rows[:types].include?("blob") blob_indices = rows[:types].each_index.select { |idx| rows[:types][idx] == "blob" } data.each_index do |idx| blob_indices.each do |bi| data[idx][bi] = Sequel::SQL::Blob.new(Tapsoob::Utils.base64decode(data[idx][bi])) unless data[idx][bi].nil? end end end # Parse date/datetime/time columns if rows.has_key?(:types) %w(date datetime time).each do |type| if rows[:types].include?(type) type_indices = rows[:types].each_index.select { |idx| rows[:types][idx] == type } data.each_index do |idx| type_indices.each do |ti| data[idx][ti] = Sequel.send("string_to_#{type}".to_sym, data[idx][ti]) unless data[idx][ti].nil? end end end end end # Remove id column if @options[:"discard-identity"] && rows[:header].include?("id") columns = rows[:header] - ["id"] data = data.map { |d| d[1..-1] } end table.import(columns, data, :commit_every => 100) rescue Exception => ex case ex. when /integer out of range/ then raise Tapsoob::InvalidData, <<-ERROR, [] \nDetected integer data that exceeds the maximum allowable size for an integer type. This generally occurs when importing from SQLite due to the fact that SQLite does not enforce maximum values on integer types. ERROR else raise ex end end |
#increment(row_count) ⇒ Object
68 69 70 |
# File 'lib/tapsoob/data_stream.rb', line 68 def increment(row_count) state[:offset] += row_count end |
#log ⇒ Object
24 25 26 27 |
# File 'lib/tapsoob/data_stream.rb', line 24 def log Tapsoob.log.level = Logger::DEBUG if state[:debug] Tapsoob.log end |
#max_chunksize_training ⇒ Object
123 124 125 |
# File 'lib/tapsoob/data_stream.rb', line 123 def max_chunksize_training 20 end |
#order_by(name = nil) ⇒ Object
61 62 63 64 65 66 |
# File 'lib/tapsoob/data_stream.rb', line 61 def order_by(name=nil) @order_by ||= begin name ||= table_name Tapsoob::Utils.order_by(db, name) end end |
#parse_encoded_data(encoded_data, checksum) ⇒ Object
193 194 195 196 197 198 199 200 201 202 203 204 205 |
# File 'lib/tapsoob/data_stream.rb', line 193 def parse_encoded_data(encoded_data, checksum) raise Tapsoob::CorruptedData.new("Checksum Failed") unless Tapsoob::Utils.valid_data?(encoded_data, checksum) begin return Marshal.load(Tapsoob::Utils.base64decode(encoded_data)) rescue Object => e unless ENV['NO_DUMP_MARSHAL_ERRORS'] puts "Error encountered loading data, wrote the data chunk to dump.#{Process.pid}.dat" File.open("dump.#{Process.pid}.dat", "w") { |f| f.write(encoded_data) } end raise e end end |
#string_columns ⇒ Object
53 54 55 |
# File 'lib/tapsoob/data_stream.rb', line 53 def string_columns @string_columns ||= Tapsoob::Utils.incorrect_blobs(db, table_name) end |
#table ⇒ Object
57 58 59 |
# File 'lib/tapsoob/data_stream.rb', line 57 def table @table ||= db[table_name_sql] end |
#table_name ⇒ Object
37 38 39 |
# File 'lib/tapsoob/data_stream.rb', line 37 def table_name state[:table_name].to_sym end |
#table_name_sql ⇒ Object
41 42 43 |
# File 'lib/tapsoob/data_stream.rb', line 41 def table_name_sql table_name end |
#to_hash ⇒ Object
45 46 47 |
# File 'lib/tapsoob/data_stream.rb', line 45 def to_hash state.merge(:klass => self.class.to_s) end |
#to_json ⇒ Object
49 50 51 |
# File 'lib/tapsoob/data_stream.rb', line 49 def to_json JSON.generate(to_hash) end |
#update_chunksize_stats ⇒ Object
def fetch_chunksize
chunksize = state[:chunksize]
return chunksize if state[:num_chunksize] < max_chunksize_training
return chunksize if state[:avg_chunksize] == 0
return chunksize if state[:error]
state[:avg_chunksize] > chunksize ? state[:avg_chunksize] : chunksize
end
135 136 137 138 139 140 |
# File 'lib/tapsoob/data_stream.rb', line 135 def update_chunksize_stats return if state[:num_chunksize] >= max_chunksize_training state[:total_chunksize] += state[:chunksize] state[:num_chunksize] += 1 state[:avg_chunksize] = state[:total_chunksize] / state[:num_chunksize] rescue state[:chunksize] end |
#verify_stream ⇒ Object
266 267 268 |
# File 'lib/tapsoob/data_stream.rb', line 266 def verify_stream state[:offset] = table.count end |